import numpy as np
from scipy.spatial.transform import Rotation as RotationScipy

from datagen.utilities.arrays import normalize

DEFAULT_WORLD_UP_VECTOR = np.array([0, 0, 1])

DEFAULT_CAMERA_ROTATION = np.array([[-1, 0, 0], [0, 0, -1], [0, -1, 0]])


class Rotation:
    def __init__(self, rotation_matrix: np.ndarray):
        self._matrix = rotation_matrix

    @property
    def matrix(self) -> np.ndarray:
        return self._matrix

    @classmethod
    def from_ypr(cls, yaw: float, pitch: float, roll: float):
        matrix = RotationScipy.from_euler("yxz", (-yaw, pitch, -roll)).as_matrix()
        return cls(matrix @ DEFAULT_CAMERA_ROTATION)

    def to_ypr(self) -> np.array:
        yaw, pitch, roll = RotationScipy.from_matrix(self._matrix @ DEFAULT_CAMERA_ROTATION.T).as_euler(
            "yxz", degrees=True
        )
        return np.array([-yaw, pitch, roll])


class Extrinsics:
    """
    This class is intentionally generic, in case we'd like
    to use it on assets other than camera (light etc.).
    """

    def __init__(self, location: np.ndarray, rotation: Rotation):
        self._location = location
        self._rotation = rotation

    @property
    def location(self) -> np.ndarray:
        return self._location

    @property
    def rotation(self) -> Rotation:
        return self._rotation

    @property
    def matrix(self) -> np.ndarray:
        rotated_location = -(self._rotation.matrix @ self._location)
        extrinsic_matrix = np.hstack([self._rotation.matrix, rotated_location[..., np.newaxis]])
        return extrinsic_matrix

    @classmethod
    def from_look_at(
        cls,
        camera_location: np.ndarray,
        look_at_point: np.ndarray,
        world_up_vector: np.ndarray = DEFAULT_WORLD_UP_VECTOR,
    ) -> "Extrinsics":
        look_at_vector = normalize(camera_location - look_at_point)
        right_camera = normalize(np.cross(world_up_vector, look_at_vector))
        up_camera = normalize(np.cross(look_at_vector, right_camera))
        rotation = np.reshape(np.concatenate([right_camera, up_camera, look_at_vector]).T, (3, 3))
        if np.linalg.det(rotation) == -1:
            rotation = -rotation
        return Extrinsics(location=camera_location, rotation=Rotation(rotation))

    @classmethod
    def from_yaw_pitch_roll(cls, camera_location: np.ndarray, ypr: np.ndarray) -> "Extrinsics":
        return Extrinsics(location=camera_location, rotation=Rotation.from_ypr(*ypr))



from abc import ABC, abstractmethod
from enum import Enum


class ImageFormat(Enum):
    PNG = "png"
    EXR = "exr"


class ImagingLibrary(ABC):
    def read(self, image_file_path: str, keep_alpha: bool, convert_to_uint8: bool):
        file_format = self._get_file_format(image_file_path)
        if file_format == ImageFormat.PNG.value:
            return self._read_png(image_file_path, keep_alpha, convert_to_uint8)
        elif file_format == ImageFormat.EXR.value:
            return self._read_exr(image_file_path)
        else:
            raise ValueError(f"Unsupported image format: {file_format}")

    @staticmethod
    def _get_file_format(image_file_path: str) -> str:
        return image_file_path.split(".")[-1]

    @abstractmethod
    def _read_png(self, image_file_path: str, keep_alpha: bool, convert_to_uint8: bool): ...

    @abstractmethod
    def _read_exr(self, image_file_path: str): ...



from dataclasses import field
from typing import TypeVar, Optional, Generator, Iterable

import marshmallow
import marshmallow_dataclass
from marshmallow import pre_load, ValidationError
from marshmallow.fields import Field

from datagen.modalities.textual.common.ndarray import NumpyArray

SubSegments = TypeVar("SubSegments")


@marshmallow_dataclass.dataclass
class Keypoint:
    name: str
    coords_2d: NumpyArray = field(repr=False)
    coords_3d: NumpyArray = field(repr=False)

    @pre_load
    def rearrange_fields(self, in_data: dict, **kwargs) -> dict:
        if all(key in in_data.keys() for key in ["global_3d", "pixel_2d"]):
            in_data["coords_3d"] = in_data.pop("global_3d")
            in_data["coords_2d"] = in_data.pop("pixel_2d")
        return in_data


class SubSegmentsField(Field):
    def _deserialize(self, value, *args, **kwargs):
        sub_segments = []
        for name, data in value.items():
            try:
                sub_segments.append(Keypoint.Schema().load({"name": name, **data}))
            except ValidationError:
                sub_segments.append(NestedSegment(name, self._deserialize(data, *args, **kwargs)))
        return sub_segments


class KeypointsSchema(marshmallow.Schema):
    TYPE_MAPPING = {SubSegments: SubSegmentsField}


@marshmallow_dataclass.dataclass(base_schema=KeypointsSchema)
class NestedSegment:
    name: str
    sub_segments: Optional[SubSegments]

    def __getattr__(self, item):
        for sub_seg in self.sub_segments:
            if sub_seg.name == item:
                return sub_seg

    def __dir__(self) -> Iterable[str]:
        return [seg.name for seg in self.sub_segments]


@marshmallow_dataclass.dataclass(base_schema=KeypointsSchema)
class Keypoints:
    scene: SubSegments

    @pre_load
    def rearrange_fields(self, in_data: dict, **kwargs) -> dict:
        return {"scene": _convert_multi_keypoints_segments_to_matrices(in_data)}

    def __getattr__(self, item):
        for sub_seg in self.scene:
            if sub_seg.name == item:
                return sub_seg

    def __dir__(self) -> Iterable[str]:
        return [seg.name for seg in self.scene]


def _convert_multi_keypoints_segments_to_matrices(in_data: dict) -> dict:
    converted_dict = {}
    for name, data in in_data.items():
        if isinstance(data, dict):
            if _is_multi_keypoints_segment(data):
                converted_dict[name] = _convert_to_matrices(data)
            else:
                converted_dict[name] = _convert_multi_keypoints_segments_to_matrices(data)
        else:
            converted_dict[name] = data
    return converted_dict


def _is_multi_keypoints_segment(in_data: dict) -> bool:
    return all(s.isnumeric() for s in in_data.keys())


def _convert_to_matrices(kp_num_to_coords: dict) -> dict:
    coords_2d_matrix, coords_3d_matrix = [], []
    for kp_coords in _get_coords_sorted_by_kp_num(kp_num_to_coords):
        coords_2d, coords_3d = kp_coords["pixel_2d"], kp_coords["global_3d"]
        coords_2d_matrix.append([coords_2d["x"], coords_2d["y"]])
        coords_3d_matrix.append([coords_3d["x"], coords_3d["y"], coords_3d["z"]])
    return {"coords_2d": coords_2d_matrix, "coords_3d": coords_3d_matrix}


def _get_coords_sorted_by_kp_num(kp_num_to_coords_nd) -> Generator:
    for keypoint_num in sorted(int(s) for s in kp_num_to_coords_nd.keys()):
        yield kp_num_to_coords_nd[str(keypoint_num)]



import json
from pathlib import Path
from typing import List, Optional

from datagen.api.assets import Background, Camera, DataRequest, Glasses, Human, HumanDatapoint, Light, Mask
from datagen.api.client.impl import DataGenerationClient
from datagen.api.client.schemas import DataResponse, DataResponseStatus
from datagen.api.requests.datapoint.builder import HumanDatapointBuilder
from datagen.api.requests.director import DataRequestDirector
from datagen.dev.logging import get_logger

logger = get_logger(__name__)

DEFAULT_DATAPOINT_REQUESTS_JSON_NAME = "datagen_data_request.json"


class DatagenAPI:
    def __init__(self, client: DataGenerationClient):
        self._request_director = DataRequestDirector()
        self._client = client

    def create_datapoint(
        self,
        human: Human,
        camera: Camera,
        glasses: Optional[Glasses] = None,
        mask: Optional[Mask] = None,
        background: Optional[Background] = None,
        lights: Optional[List[Light]] = None,
    ) -> HumanDatapoint:
        self._request_director.builder = HumanDatapointBuilder(human, camera, glasses, mask, background, lights)
        return self._request_director.build_datapoint()

    def generate(self, request: DataRequest) -> DataResponse:
        return self._client.generate(request)

    def stop(self, generation_id: str) -> None:
        return self._client.stop(generation_id=generation_id)

    def get_status(self, generation_id: str) -> DataResponseStatus:
        return self._client.get_status(generation_id=generation_id)

    def get_download_urls(self, generation_id: str) -> List[str]:
        return self._client.get_download_urls(generation_id=generation_id)

    def download(self, urls: List[str], dest: str) -> None:
        self._client.download(urls=urls, dest=Path(dest))

    def load(self, path: str) -> DataRequest:
        path_ = self._get_request_json_path(path)
        return DataRequest.parse_file(path_)

    def dump(self, request: DataRequest, path: Optional[str] = None) -> None:
        path_ = self._get_request_json_path(path)
        path_.write_text(json.dumps(request.dict(exclude={"attributes", "presets"}), indent=3, sort_keys=True))
        logger.info(
            f"Data request was successfully dumped to path '{str(path_.absolute())}' "
            + f"({len(request.data)} datapoints total)."
            ""
        )

    def _get_request_json_path(self, path: Optional[str]) -> Path:
        path_ = Path(path) if path else Path.cwd().joinpath(DEFAULT_DATAPOINT_REQUESTS_JSON_NAME)
        if not path_.parent.exists():
            raise FileNotFoundError(f"{path_.parent} folder does not exist, cannot dump requests")
        return path_



import math
import sys
from dataclasses import dataclass
from typing import Union, ClassVar, NamedTuple, TypeVar, Generator, List

import numpy as np

from datagen import config
from datagen.dev import FunctionalModule
from datagen.api import assets
from datagen.utilities.arrays import to_arrays
from datagen.utilities.extrinsics import Extrinsics

Resolution = NamedTuple('Resolution', x=int, y=int)

Sensor = NamedTuple("Sensor", width=float, height=float)


@dataclass
class Focus:
    focal_length: float
    fov: float

    @classmethod
    def from_focal_length(cls, focal_length: float, sensor_width: float) -> "Focus":
        fov = 2 * np.arctan2(0.5 * sensor_width, focal_length)
        return cls(fov=fov, focal_length=focal_length)

    @classmethod
    def from_fov(cls, fov: float, sensor_width: float) -> "Focus":
        focal_length = sensor_width / (np.tan(fov / 2) * 2)
        return cls(fov=fov, focal_length=focal_length)


@dataclass
class Intrinsics:
    resolution: Resolution
    sensor: Sensor
    focus: Focus
    pxl_aspect_ratio: float

    @property
    def matrix(self):
        pixels_per_mm_h = self.resolution.x / self.sensor.height
        pixels_per_mm_w = self.resolution.y / self.sensor.width
        f_x, f_y = self.focus.focal_length * pixels_per_mm_h, self.focus.focal_length * pixels_per_mm_w
        return np.array(
            [
                [f_x, 0, self.resolution.x / 2],
                [0.0, f_y * self.pxl_aspect_ratio, self.resolution.y / 2],
                [0.0, 0.0, 1.0],
            ]
        )


P = TypeVar("P", bound=Union[assets.Point, "np.ndarray"])
CP = TypeVar("CP", bound=Union[assets.Camera, assets.Point, "np.ndarray"])


class CameraUtils:
    """
    Camera Asset Utilities
    """

    PLUGINS_GROUP_NAME: ClassVar[str] = "datagen.plugins.utils.camera"

    def calculate_rotation(self, camera: CP, look_at_point: P) -> assets.Rotation:
        """Calculates rotation needed in order to rotate the camera towards a given look-at-point.

        Args:
            camera (Union[assets.Camera, assets.Point, np.ndarray]): The camera to calculate the rotation for.
            look_at_point (Union[assets.Point, np.ndarray]): The look-at-point the camera should be rotated towards.

        Returns:
            assets.Rotation: The yaw-pitch-roll value needed in order to rotate a camera
            towards the given look-at-point.

        Examples:
            >>> camera_utils.calculate_rotation(
            >>>     camera=np.ndarray([-0.056, -1.985, -0.367]), look_at_point=np.ndarray([0., 0., 0.1]))
            >>> )
            np.ndarray([ 1.641, 13.246,  0.])

            >>> camera_utils.calculate_rotation(
            >>>     camera=assets.Point(x=-0.056, y=-1.985, z=-0.367), look_at_point=assets.Point(x=0., y=0., z=0.1))
            >>> )
            np.ndarray([ 1.641, 13.246,  0.])

            >>> camera = assets.Camera()
            >>> camera.extrinsics.location = assets.Point(x=-0.056, y=-1.985, z=-0.367)
            >>> camera_utils.calculate_rotation(camera, look_at_point=assets.Point(x=0., y=0., z=0.1)))
            Point(x=1.641, y=13.246,  z=0.)
        """
        camera_location = self._get_camera_location(camera)
        camera_location, look_at_point = to_arrays(camera_location, look_at_point)
        extrinsics = Extrinsics.from_look_at(camera_location, look_at_point)
        ypr = extrinsics.rotation.to_ypr()
        return assets.Rotation.from_ypr(ypr)

    def rotate(self, camera: assets.Camera, look_at_point: P) -> None:
        """Rotates the camera towards the given look-at-point.

        Args:
            camera (assets.Camera): The camera to rotate.
            look_at_point (Union[assets.Point, np.ndarray]): The look-at-point the camera should be rotated towards.

        Examples:
            >>> camera = assets.Camera()
            >>> camera.extrinsics.location = assets.Point(x=-0.056, y=-1.985, z=-0.367)
            >>> camera_utils.rotate(camera, look_at_point=assets.Point(x=0., y=0., z=0.1)))
            >>> camera.extrinsics.location
            Point(x=1.641, y=13.246,  z=0.)
        """
        camera.extrinsics.rotation = self.calculate_rotation(camera, look_at_point=look_at_point)

    def _get_camera_location(self, camera: Union[assets.Camera, P]) -> P:
        return camera if isinstance(camera, (assets.Point, np.ndarray)) else camera.extrinsics.location

    def get_extrinsic_matrix(self, camera: assets.Camera) -> "np.ndarray":
        """Calculates the extrinsic matrix of the camera.

        Args:
            camera (assets.Camera): The camera to get extrinsic matrix for.

        Returns:
            np.ndarray: The extrinsic matrix of the camera
        """
        extrinsics = self._get_extrinsics_from_camera_asset(camera)
        return extrinsics.matrix

    def get_rotation_matrix(self, camera: assets.Camera) -> "np.ndarray":
        """Calculates the rotation matrix of the camera.

        Args:
            camera (assets.Camera): The camera to get rotation matrix for.

        Returns:
            np.ndarray: The rotation matrix of the camera
        """
        extrinsics = self._get_extrinsics_from_camera_asset(camera)
        return extrinsics.rotation.matrix

    def _get_extrinsics_from_camera_asset(self, camera: assets.Camera) -> Extrinsics:
        camera_location, yaw_pitch_roll = to_arrays(camera.extrinsics.location, camera.extrinsics.rotation)
        extrinsics = Extrinsics.from_yaw_pitch_roll(camera_location, yaw_pitch_roll)
        return extrinsics

    def create_cameras_ring(
        self,
        cameras_num: int,
        cameras_intrinsics: assets.IntrinsicParams,
        cameras_look_at_point: assets.Point,
        radius: float,
        z_value: float,
    ) -> List[assets.Camera]:
        cameras = []
        for camera_idx, camera_location in enumerate(
            self._get_ring_cameras_locations(cameras_num, cameras_look_at_point, radius, z_value)
        ):
            camera = self.init_camera(
                camera_idx=camera_idx, camera_intrinsics=cameras_intrinsics, camera_location=camera_location
            )
            self.rotate(camera, look_at_point=cameras_look_at_point)
            cameras.append(camera)
        return cameras

    def _get_ring_cameras_locations(
        self, cameras_num: int, ring_center: assets.Point, r: float, z: float
    ) -> Generator[assets.Point, None, None]:
        for i, xy_angle in enumerate(np.linspace(0, 360, num=cameras_num, endpoint=False)):
            xy_rad = (xy_angle * math.pi) / 180
            yield assets.Point(
                x=(math.sin(xy_rad) + ring_center.x) * (-1.0 * r),
                y=(math.cos(xy_rad) + ring_center.y) * (-1.0 * r),
                z=z + 0.12,
            )

    def init_camera(
        self,
        camera_idx: int = None,
        camera_intrinsics: assets.IntrinsicParams = None,
        camera_extrinsics: assets.ExtrinsicParams = None,
        camera_location: assets.Point = None,
    ) -> assets.Camera:
        camera = assets.Camera()
        camera.name = f"Camera_{str(camera_idx).zfill(2)}" if camera_idx else config.assets.camera.name
        if camera_intrinsics:
            camera.intrinsics = camera_intrinsics
        if camera_extrinsics:
            camera.extrinsics = camera_extrinsics
        if camera_location:
            camera.extrinsics.location = camera_location
        return camera


sys.modules[__name__] = FunctionalModule(functionality=CameraUtils())



import base64
import os

from dependency_injector import containers, providers

from datagen.api.client.impl import AuthenticationConfig, DataGenerationClient
from datagen.api.impl import DatagenAPI
from datagen.dev.logging import get_logger

logger = get_logger(__name__)


def token_setup_instructions() -> str:
    if os.name == "nt":  # Windows
        return "set DG_AUTH_TOKEN=<your-auth-token>"
    else:  # Linux and MacOS
        return "export DG_AUTH_TOKEN=<your-auth-token>"


def build_config() -> AuthenticationConfig:
    token = os.environ["DG_AUTH_TOKEN"]
    if not token:
        logger.error(
            f"Authentication token is uninitialized. You can create a new token at "
            f"https://app.prod.datagen.tech/token. After creating the token insert the following:"
            f" {token_setup_instructions()}"
        )
        raise Exception("Authentication token is uninitialized.")
    _, token_id = base64.b64decode(token).decode("utf-8").split(".")[:2]
    user_id, org_id = tuple(base64.b64decode(token_id).decode("utf-8").split(".")[:2])

    return AuthenticationConfig(token=token, user_id=user_id, org_id=org_id)


class ApiContainer(containers.DeclarativeContainer):

    config = providers.Callable(build_config)

    client = providers.Singleton(DataGenerationClient, config=config)

    api = providers.Singleton(DatagenAPI, client=client)



from functools import partial

from dependency_injector import containers, providers

from datagen.dev import load_resource
from datagen.api import assets
from datagen.api.catalog import attributes
from datagen.api.catalog.hooks import HumansDefaultsHook, HICPresetsHook
from datagen.api.catalog.impl import AssetCatalog, DatagenAssetsCatalog, BackgroundsCatalog

load_cache_resource = partial(load_resource, "cache")


class AssetsCatalogContainer(containers.DeclarativeContainer):

    humans = providers.Singleton(
        AssetCatalog,
        asset_type=assets.Human,
        asset_id_to_asset_attrs=load_cache_resource("humans", "attributes.json"),
        hooks=providers.List(
            providers.Singleton(
                HumansDefaultsHook, asset_id_to_asset_defaults=load_cache_resource("humans", "defaults.json")
            )
        ),
    )

    sequences = providers.Singleton(
        AssetCatalog,
        asset_type=assets.DataSequence,
        asset_id_to_asset_attrs=load_cache_resource("hic", "attributes.json"),
        hooks=providers.List(
            providers.Singleton(HICPresetsHook, asset_id_to_asset_presets=load_cache_resource("hic", "presets.json"))
        ),
    )
    eyes = providers.Singleton(
        AssetCatalog,
        asset_type=assets.Eyes,
        asset_id_to_asset_attrs=load_cache_resource("eyes.json"),
    )

    hair = providers.Singleton(
        AssetCatalog,
        asset_type=assets.Hair,
        asset_id_to_asset_attrs=load_cache_resource("hair.json"),
    )

    eyebrows = providers.Singleton(
        AssetCatalog,
        asset_type=assets.Eyebrows,
        asset_id_to_asset_attrs=load_cache_resource("eyebrows.json"),
    )

    facial_hair = providers.Singleton(
        AssetCatalog,
        asset_type=assets.FacialHair,
        asset_id_to_asset_attrs=load_cache_resource("beards.json"),
    )

    glasses = providers.Singleton(
        AssetCatalog,
        asset_type=assets.Glasses,
        asset_id_to_asset_attrs=load_cache_resource("glasses.json"),
    )

    masks = providers.Singleton(
        AssetCatalog,
        asset_type=assets.Mask,
        asset_id_to_asset_attrs=load_cache_resource("masks.json"),
    )

    backgrounds = providers.Singleton(
        BackgroundsCatalog,
        asset_type=assets.Background,
        asset_id_to_asset_attrs=load_cache_resource("backgrounds.json"),
    )

    catalog = providers.Singleton(
        DatagenAssetsCatalog,
        humans=humans,
        sequences=sequences,
        hair=hair,
        eyes=eyes,
        eyebrows=eyebrows,
        facial_hair=facial_hair,
        glasses=glasses,
        masks=masks,
        backgrounds=backgrounds,
    )



import abc
from dataclasses import dataclass
from typing import Type, Optional


class ModalityFileNotFoundError(RuntimeError):
    ...


@dataclass
class Modality(abc.ABC):
    file_name: str


class ModalityDescriptor(abc.ABC):
    def __init__(self, fget=None, **kwargs):
        self._fget = fget

    def __get__(self, dp, *args):
        modality = self._fget(dp)
        modality_file_path = self._get_modality_file_path(dp, modality)
        return self._read(dp, modality, modality_file_path)

    @staticmethod
    def _get_modality_file_path(dp, modality: Modality) -> str:
        modality_file_path = None
        camera_modality_path = dp.camera_path.joinpath(modality.file_name)
        frame_modality_path = dp.frame_path.joinpath(modality.file_name)
        scene_modality_path = dp.scene_path.joinpath(modality.file_name)
        if camera_modality_path.exists():
            modality_file_path = camera_modality_path
        elif frame_modality_path.exists():
            modality_file_path = frame_modality_path
        elif scene_modality_path.exists():
            modality_file_path = scene_modality_path
        return str(modality_file_path) if modality_file_path is not None else None

    @abc.abstractmethod
    def _read(self, dp, modality: Modality, modality_file_path: str):
        ...


class ModalityPreProcess(abc.ABC):
    @abc.abstractmethod
    def __call__(self):
        ...


@dataclass
class TextualModality(Modality):
    factory_name: str
    pre_process: Optional[ModalityPreProcess] = None


class TextualModalityDescriptor(ModalityDescriptor):
    def _read(self, dp, modality: TextualModality, modality_file_path: str):
        if modality.pre_process is not None:
            modality.pre_process()
        if modality_file_path is None:
            ModalityFileNotFoundError(f"'{modality.file_name}' not found for datapoint {dp}")
        else:
            return dp.modalities_container.read_textual_modality(
                modality_file_path=modality_file_path, modality_factory_name=modality.factory_name
            )


@dataclass
class VisualModality(Modality):
    keep_alpha: bool = False
    convert_to_uint8: bool = False


class VisualModalityDescriptor(ModalityDescriptor):
    def _read(self, dp, modality: VisualModality, modality_file_path: str):
        if modality_file_path is None:
            return None
        else:
            return dp.modalities_container.read_visual_modality(
                modality_file_path=modality_file_path,
                keep_alpha=modality.keep_alpha,
                convert_to_uint8=modality.convert_to_uint8,
            )



import json
import os
from dataclasses import dataclass
from pathlib import Path
from typing import ClassVar

from datagen import modalities
from datagen.api.assets import HumanDatapoint
from datagen.components.datapoint.entity import base


@dataclass
class DataPoint(base.DataPoint):
    PLUGINS_GROUP_NAME: ClassVar[str] = "datagen.plugins.datapoints.identities"

    @modalities.textual_modality
    def actor_metadata(self) -> modalities.TextualModality:
        return modalities.TextualModality(factory_name="actor_metadata", file_name="actor_metadata.json")

    @modalities.textual_modality
    def face_bounding_box(self) -> modalities.TextualModality:
        return modalities.TextualModality(factory_name="face_bounding_box", file_name="face_bounding_box.json")

    @modalities.textual_modality
    def keypoints(self) -> modalities.TextualModality:
        return modalities.TextualModality(
            factory_name="keypoints",
            file_name=os.path.join("key_points", "all_key_points.json"),
            pre_process=CreateV1AllKeypointsFile(self),
        )

    @modalities.textual_modality
    def _datapoint_request(self) -> modalities.TextualModality:
        """
        :returns: DataRequest containing the HumanDatapoint asset associated with this datapoint.
        """
        return modalities.TextualModality(factory_name="datapoint_request", file_name="datapoint_request.json")

    @property
    def datapoint_request(self) -> HumanDatapoint:
        return self._datapoint_request.data[0]


class CreateV1AllKeypointsFile(modalities.ModalityPreProcess):
    """
    V1 keypoints content comes in two separate files: standard_keypoints.json & dense_keypoints.json
    which have to be consolidated into a single json since the SDK assumes that the content of
    any given modality will always appear in a single json file. We consolidate both files into
    '<DATAPOINT_CAMERA_PATH>/key_points/all_key_points.json', To match v2 keypoints json file location.
    """

    def __init__(self, dp: DataPoint):
        self._dp = dp

    def __call__(self):
        is_v1_keypoints = not self._get_all_keypoints_file().exists()
        if is_v1_keypoints:
            self._create_all_keypoints_file_for_v1()

    def _create_all_keypoints_file_for_v1(self):
        self._get_all_keypoints_file().parent.mkdir()
        self._get_all_keypoints_file().write_text(self._get_keypoints_v1_file_content())

    def _get_keypoints_v1_file_content(self) -> str:
        standard_keypoints = json.loads(self._dp.camera_path.joinpath("standard_keypoints.json").read_text())
        dense_keypoints = json.loads(self._dp.camera_path.joinpath("dense_keypoints.json").read_text())
        keypoints_v1_file_content = json.dumps(
            {"standard": standard_keypoints, "dense": dense_keypoints}, indent=4, sort_keys=True
        )
        return keypoints_v1_file_content

    def _get_all_keypoints_file(self) -> Path:
        return self._dp.camera_path.joinpath("key_points", "all_key_points.json")



from datagen.modalities.descriptors import (
    VisualModalityDescriptor,
    VisualModality,
    TextualModalityDescriptor,
    TextualModality,
    ModalityPreProcess,
)

visual_modality = VisualModalityDescriptor
textual_modality = TextualModalityDescriptor



from dataclasses import dataclass

from datagen.components import SourcesRepository
from datagen.components.dataset import Dataset, DatasetConfig

DEFAULT_DATASET_CONFIG = DatasetConfig(imaging_library="opencv")


@dataclass
class Datagen:
    def load(self, *dataset_sources: str, dataset_config: DatasetConfig = DEFAULT_DATASET_CONFIG) -> Dataset:
        sources_repo = SourcesRepository(dataset_sources)
        return Dataset(sources_repo=sources_repo, config=dataset_config)



from dataclasses import dataclass
from typing import Optional, Union, List

from datagen.api.assets import (
    Human,
    Camera,
    Glasses,
    Mask,
    Background,
    Light,
    HumanDatapoint,
    Accessories,
)


@dataclass
class HumanDatapointBuilder:
    human: Human
    camera: Camera
    glasses: Optional[Glasses]
    mask: Optional[Mask]
    background: Optional[Background]
    lights: Optional[List[Light]]

    def get_basic_datapoint(self) -> HumanDatapoint:
        return HumanDatapoint(human=self.human.copy(), camera=self.camera.copy())

    def get_accessories(self) -> Union[Accessories, None]:
        if self.glasses is None and self.mask is None:
            return None
        else:
            accessories = Accessories()
            if self.glasses is not None:
                accessories.glasses = self.glasses.copy()
            if self.mask is not None:
                accessories.mask = self.mask.copy()
            return accessories

    def get_background(self) -> Union[Background, None]:
        if self.background is not None:
            return self.background.copy()
        else:
            return None

    def get_lights(self) -> Union[List[Light], None]:
        if self.lights is not None and len(self.lights) > 0:
            return [l_.copy() for l_ in self.lights]
        else:
            return None



import sys

from datagen.api import ApiContainer
from datagen.dev import FunctionalModule

print("❗❗❗'datagen.api.datapoint' module is deprecated, please use 'datagen.api' instead.")

api = ApiContainer.api()

sys.modules[__name__] = FunctionalModule(functionality=api)



from datagen_protocol.schema.assets.attributes import *


class AllOf(list):
    def __init__(self, *attributes):
        super().__init__(attributes)


class AnyOf(list):
    def __init__(self, *attributes):
        super().__init__(attributes)


class Exactly(list):
    def __init__(self, *attributes):
        super().__init__(attributes)



from pathlib import Path
from dependency_injector import containers, providers

from datagen.imaging.opencv import OpenCVImagingLibrary
from datagen.modalities import textual as textual_modalities


class VisualModalitiesContainer(containers.DeclarativeContainer):

    config = providers.Configuration()

    imaging_library = providers.Selector(config, opencv=providers.Singleton(OpenCVImagingLibrary))


def read_visual_modality(
    modalities_container: containers.DeclarativeContainer,
    modality_file_path: str,
    keep_alpha: bool,
    convert_to_uint8: bool,
):
    return (
        modalities_container.visual()
        .imaging_library()
        .read(image_file_path=modality_file_path, keep_alpha=keep_alpha, convert_to_uint8=convert_to_uint8)
    )


def read_textual_modality(
    modalities_container: containers.DeclarativeContainer, modality_file_path: str, modality_factory_name: str
):
    return modalities_container.textual().providers[modality_factory_name](modality_file_path=modality_file_path)


class DatapointModalitiesContainer(containers.DeclarativeContainer):

    __self__ = providers.Self()

    config = providers.Configuration()

    visual = providers.Container(VisualModalitiesContainer, config=config.imaging_library)

    textual = providers.Selector(
        config.environment,
        hic=providers.Container(textual_modalities.HICModalitiesContainer),
        identities=providers.Container(textual_modalities.IdentitiesModalitiesContainer),
    )

    read_visual_modality = providers.Callable(read_visual_modality, modalities_container=__self__)

    read_textual_modality = providers.Callable(read_textual_modality, modalities_container=__self__)

    wiring_config = containers.WiringConfiguration(packages=[textual_modalities])



import marshmallow_dataclass
from marshmallow import pre_load

from datagen.modalities.textual.identities.keypoints import base


@marshmallow_dataclass.dataclass(base_schema=base.KeypointsSchema)
class SceneKeypoints(base.SceneKeypoints):
    @pre_load
    def convert_to_v2_structure(self, in_data: dict, **kwargs) -> dict:
        in_data["standard"]["coords_2d"] = in_data["standard"].pop("keypoints_2d_coordinates")
        in_data["standard"]["coords_3d"] = in_data["standard"].pop("keypoints_3d_coordinates")
        in_data["dense"]["coords_2d"] = in_data["dense"].pop("keypoints_2d_coordinates")
        in_data["dense"]["coords_3d"] = in_data["dense"].pop("keypoints_3d_coordinates")
        return {"scene": {"face": in_data}}

